/**
 * Copyright (c) 2013-2015 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see <http://www.gnu.org/licenses/>
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.cli.extension.command.monitoring.so.components.framework;

import java.net.MalformedURLException;
import java.net.URL;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.Properties;

import org.apache.commons.cli.CommandLine;
import org.apache.commons.cli.CommandLineParser;
import org.apache.commons.cli.MissingArgumentException;
import org.apache.commons.cli.MissingOptionException;
import org.apache.commons.cli.Option;
import org.apache.commons.cli.OptionBuilder;
import org.apache.commons.cli.Options;
import org.apache.commons.cli.ParseException;
import org.apache.commons.cli.PosixParser;
import org.apache.commons.cli.UnrecognizedOptionException;
import org.ow2.petals.admin.jmx.JMXClientConnection;
import org.ow2.petals.cli.api.shell.Shell;
import org.ow2.petals.cli.extension.command.monitoring.so.DefectEchoer;
import org.ow2.petals.cli.extension.command.monitoring.so.NagiosMapper;
import org.ow2.petals.cli.extension.command.monitoring.so.api.EmittableDefect;
import org.ow2.petals.cli.extension.command.monitoring.so.api.SubscriptionObject;
import org.ow2.petals.cli.extension.command.monitoring.so.api.exception.SubscriptionObjectBadArgumentNumberException;
import org.ow2.petals.cli.extension.command.monitoring.so.api.exception.SubscriptionObjectException;
import org.ow2.petals.cli.extension.command.monitoring.so.api.exception.SubscriptionObjectInvalidException;
import org.ow2.petals.cli.extension.command.monitoring.so.api.exception.SubscriptionObjectMissingArgumentException;
import org.ow2.petals.cli.extension.command.monitoring.so.api.exception.SubscriptionObjectMissingOptionsException;
import org.ow2.petals.cli.extension.command.monitoring.so.components.framework.defect.MessageExchangeAcceptorDead;
import org.ow2.petals.cli.extension.command.monitoring.so.components.framework.defect.NoMoreMessageExchangeAcceptorThread;
import org.ow2.petals.cli.extension.command.monitoring.so.components.framework.defect.NoMoreMessageExchangeProcessorObject;
import org.ow2.petals.cli.extension.command.monitoring.so.components.framework.defect.NoMoreMessageExchangeProcessorThread;
import org.ow2.petals.cli.extension.command.monitoring.so.components.framework.exception.ComponentDoesNotExistException;
import org.ow2.petals.jmx.api.api.JMXClient;
import org.ow2.petals.jmx.api.api.exception.ConnectionErrorException;
import org.ow2.petals.jmx.api.api.monitoring.exception.DefectEmitterDoesNotExistsException;
import org.ow2.petals.jmx.api.api.monitoring.exception.DefectListenerException;
import org.ow2.petals.jmx.api.api.monitoring.exception.NoDefectListenerException;

/**
 * The subscription object associated to the Petals component
 * 
 * @author Christophe DENEUX - Linagora
 * 
 */
public abstract class AbstractMonitoredComponent implements SubscriptionObject {

    private final static EmittableDefect[] CDK_EMITTABLE_DEFECTS = new EmittableDefect[] {
            new MessageExchangeAcceptorDead(), new NoMoreMessageExchangeProcessorObject(),
            new NoMoreMessageExchangeAcceptorThread(), new NoMoreMessageExchangeProcessorThread() };

    /**
     * The option to get the component name
     */
    public static final String COMPONENT_NAME_LONG_OPTION = "component-name";

    public static final String COMPONENT_NAME_SHORT_OPTION = "n";

    private static final Option COMPONENT_NAME_OPTION = OptionBuilder.isRequired(true).hasArgs(1)
            .withArgName("component-name").withDescription("The identifier of the component.")
            .withLongOpt(COMPONENT_NAME_LONG_OPTION).create(COMPONENT_NAME_SHORT_OPTION);

    /**
     * The option to get the Nagios mapping file URL
     */
    public static final String MAPPING_FILE_URL_LONG_OPTION = "mapping-file-url";

    public static final String MAPPING_FILE_URL_SHORT_OPTION = "t";

    private static final Option MAPPING_FILE_URL_OPTION = OptionBuilder.isRequired(false)
            .hasArgs(1).withArgName("mapping-file-url")
            .withDescription("The Nagios mapping file URL.")
            .withLongOpt(MAPPING_FILE_URL_LONG_OPTION).create(MAPPING_FILE_URL_SHORT_OPTION);

    /**
     * The subscription object options
     */
    private Options options;

    /**
     * The shell in which the sub-function is running
     */
    private Shell shell;

    public AbstractMonitoredComponent() {
        this.createOptions();
    }

    private void createOptions() {
        this.options = new Options();
        this.options.addOption(COMPONENT_NAME_OPTION);
        this.options.addOption(MAPPING_FILE_URL_OPTION);
    }

    @Override
    public Options getOptions() {
        return this.options;
    }

    @Override
    public void resetOptions() {
        this.createOptions();
    }

    @Override
    public void setShell(final Shell shell) {
        this.shell = shell;
    }

    @Override
    public void subscribe(final String[] args) throws SubscriptionObjectBadArgumentNumberException,
            SubscriptionObjectMissingArgumentException, SubscriptionObjectMissingOptionsException,
            SubscriptionObjectException {

        final CommandLineParser parser = new PosixParser();
        try {
            final CommandLine cli = parser.parse(this.options, args);

            // No extra option is accepted
            if (cli.getArgs().length > 0) {
                throw new SubscriptionObjectBadArgumentNumberException(this);
            }

            final String componentIdentifier = cli.getOptionValue(COMPONENT_NAME_SHORT_OPTION);

            final Properties mappingRules = new Properties();
            if (cli.hasOption(MAPPING_FILE_URL_SHORT_OPTION)) {
                final URL mappingFileURL = new URL(
                        cli.getOptionValue(MAPPING_FILE_URL_SHORT_OPTION));
                try {
                    mappingRules.load(mappingFileURL.openStream());
                } catch (final Exception e) {
                    throw new SubscriptionObjectException(this,
                            "An error occurs reading the mapping rule file", e);
                }
            }

            final JMXClient client = JMXClientConnection.getJMXClient();

            try {
                client.subscribeToComponentMonitoringService(componentIdentifier, new DefectEchoer(
                        new NagiosMapper(mappingRules)),
                        // TODO: Add a defect filter support
                        null);
                this.shell.addAsynchronousCommand();

            } catch (final DefectEmitterDoesNotExistsException e) {
                throw new ComponentDoesNotExistException(this, componentIdentifier);
            }
        } catch (final UnrecognizedOptionException e) {
            throw new SubscriptionObjectBadArgumentNumberException(this);
        } catch (final MissingArgumentException e) {
            throw new SubscriptionObjectMissingArgumentException(this, e.getOption());
        } catch (final MissingOptionException e) {
            throw new SubscriptionObjectMissingOptionsException(this, e.getMissingOptions());
        } catch (final ParseException e) {
            throw new SubscriptionObjectInvalidException(this, e);
        } catch (final ConnectionErrorException e) {
            throw new SubscriptionObjectException(this, e);
        } catch (final MalformedURLException e) {
            throw new SubscriptionObjectException(this, e);
        } catch (final DefectListenerException e) {
            throw new SubscriptionObjectException(this, e);
        }
    }

    @Override
    public void unsubscribe(final String[] args)
            throws SubscriptionObjectBadArgumentNumberException, SubscriptionObjectException {

        final CommandLineParser parser = new PosixParser();
        try {
            final CommandLine cli = parser.parse(this.options, args);

            // No extra option is accepted
            if (cli.getArgs().length > 0 || cli.hasOption(MAPPING_FILE_URL_SHORT_OPTION)) {
                throw new SubscriptionObjectBadArgumentNumberException(this);
            }

            final String componentIdentifier = cli.getOptionValue(COMPONENT_NAME_SHORT_OPTION);

            final JMXClient client = JMXClientConnection.getJMXClient();

            try {
                client.unsubscribeToComponentMonitoringService(componentIdentifier);
                this.shell.removeAsynchronousCommand();

            } catch (final DefectEmitterDoesNotExistsException e) {
                throw new ComponentDoesNotExistException(this, componentIdentifier);
            }
        } catch (final UnrecognizedOptionException e) {
            throw new SubscriptionObjectBadArgumentNumberException(this);
        } catch (final MissingArgumentException e) {
            throw new SubscriptionObjectMissingArgumentException(this, e.getOption());
        } catch (final MissingOptionException e) {
            throw new SubscriptionObjectMissingOptionsException(this, e.getMissingOptions());
        } catch (final ParseException e) {
            throw new SubscriptionObjectInvalidException(this, e);
        } catch (final ConnectionErrorException e) {
            throw new SubscriptionObjectException(this, e);
        } catch (final NoDefectListenerException e) {
            throw new SubscriptionObjectException(this, e);
        } catch (final DefectListenerException e) {
            throw new SubscriptionObjectException(this, e);
        }
    }

    @Override
    public List<EmittableDefect> getEmittableDefects() {
        final List<EmittableDefect> emittableDefects = new ArrayList<EmittableDefect>();
        emittableDefects.addAll(Arrays.asList(CDK_EMITTABLE_DEFECTS));
        final List<EmittableDefect> dedicatedEmittableDefects = this.getDedicatedEmittableDefects();
        if (dedicatedEmittableDefects != null) {
            emittableDefects.addAll(dedicatedEmittableDefects);
        }
        return emittableDefects;
    }

    /**
     * @return the list of dedicated emittable defects of the component provided
     *         by the monitoring object. Can be <code>null</code> is no
     *         dedicated emittable defect is provided.
     */
    public abstract List<EmittableDefect> getDedicatedEmittableDefects();
}
